use std::path::Path;

use log::debug;
use sqlx::{migrate::MigrateDatabase, Sqlite, SqlitePool};

pub struct Database {
    pub connection: SqlitePool,
}

impl Database {
    pub async fn setup(database_path: impl AsRef<Path>) -> sqlx::Result<Self> {
        let connection =
            Database::create_connection(database_path.as_ref().to_str().unwrap()).await?;
        sqlx::migrate!("./db-migrations").run(&connection).await?;
        Ok(Database { connection })
    }

    async fn create_connection(database_path: &str) -> sqlx::Result<SqlitePool> {
        if !Sqlite::database_exists(database_path)
            .await
            .unwrap_or(false)
        {
            match Sqlite::create_database(database_path).await {
                Ok(_) => debug!("Database created"),
                Err(e) => {
                    debug!("Error creating database: {}", e);
                    return Err(e);
                }
            }
        }

        let connection = SqlitePool::connect(database_path).await?;
        debug!("Connected to database");
        Ok(connection)
    }
}
