use gtk::prelude::*;
use relm4::prelude::*;

use crate::sheet::Sheet;

pub struct SheetModel {
    pub label: String,
    pub sheet: Sheet,
    visible: bool,
}

#[derive(Debug, Clone)]
pub struct OnQueryUpdate {
    pub query: String,
}

#[relm4::factory(pub)]
impl FactoryComponent for SheetModel {
    type Init = Sheet;
    type ParentWidget = gtk::ListBox;
    type CommandOutput = ();
    type Input = OnQueryUpdate;
    type Output = ();

    view! {
        #[root]
        gtk::ListBoxRow {
            #[watch]
            set_visible: self.visible,
            gtk::Box {
                set_orientation: gtk::Orientation::Vertical,
                append = &gtk::Label {
                    set_label: &self.label,
                    set_halign: gtk::Align::Start,
                    set_margin_all: 10,
                }
            }
        }
    }

    fn init_model(sheet: Self::Init, _index: &DynamicIndex, _sender: FactorySender<Self>) -> Self {
        let label = match &sheet.kind {
            crate::sheet::SheetKind::Sheet { name, .. } => name,
            crate::sheet::SheetKind::Orphan {} => sheet.pdf.get_name(),
            crate::sheet::SheetKind::Book { name, .. } => name,
        };

        SheetModel {
            label: label.to_string(),
            sheet,
            visible: true,
        }
    }

    fn update(&mut self, msg: Self::Input, _sender: FactorySender<Self>) {
        let query = msg.query.to_ascii_lowercase();
        let split = query.split_ascii_whitespace();
        let label = self.label.to_ascii_lowercase();

        self.visible = split
            .into_iter()
            .all(|query_part| label.contains(query_part));
    }
}
